package com.sztzjy.money_management.config.exception.handler;


import com.sztzjy.money_management.config.exception.UnAuthorizedException;
import com.sztzjy.money_management.util.ResultEntity;
import com.sztzjy.money_management.util.ThrowableUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.security.access.AccessDeniedException;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

/**
 * 全局异常处理
 */
@Slf4j
@RestControllerAdvice
public class GlobalExceptionHandler {

    @ExceptionHandler(IllegalArgumentException.class)
    public ResultEntity<String> illegalArgumentException(IllegalArgumentException e) {
        log.error(ThrowableUtil.getStackTrace(e));
        return new ResultEntity<>(HttpStatus.BAD_REQUEST, e.getMessage());
    }

    @ExceptionHandler(AccessDeniedException.class)
    public ResultEntity<String> accessDeniedException(AccessDeniedException e) {
        log.error(ThrowableUtil.getStackTrace(e));
        return new ResultEntity<>(HttpStatus.FORBIDDEN, e.getMessage());
    }

    @ExceptionHandler(UnAuthorizedException.class)
    public ResultEntity<String> UnAuthorizedException(UnAuthorizedException e) {
        log.error(ThrowableUtil.getStackTrace(e));
        return new ResultEntity<>(HttpStatus.UNAUTHORIZED, e.getMessage());
    }

    /**
     * 处理所有接口数据验证异常
     */
    @ExceptionHandler(MethodArgumentNotValidException.class)
    public ResultEntity<String> handleMethodArgumentNotValidException(MethodArgumentNotValidException e) {
        log.error(ThrowableUtil.getStackTrace(e));
        ObjectError objectError = e.getBindingResult().getAllErrors().get(0);
        String message = objectError.getDefaultMessage();
        if (objectError instanceof FieldError) {
            message = ((FieldError) objectError).getField() + ": " + message;
        }
        return new ResultEntity<>(HttpStatus.BAD_REQUEST, message);
    }

    /**
     * 处理所有不可知的异常
     *
     * @param e 未知异常
     * @return /
     */
    @ExceptionHandler(Throwable.class)
    public ResultEntity<String> nullPointerException(Throwable e) {
        log.error(ThrowableUtil.getStackTrace(e));
        return new ResultEntity<>(HttpStatus.BAD_REQUEST, e.getMessage());
    }

    /**
     * BadCredentialsException
     */
    @ExceptionHandler(BadCredentialsException.class)
    public ResultEntity<String> badCredentialsException(BadCredentialsException e) {
        // 打印堆栈信息
        String message = "坏的凭证".equals(e.getMessage()) ? "用户名或密码不正确" : e.getMessage();
        log.error(message);
        return new ResultEntity<>(HttpStatus.BAD_REQUEST, message);
    }


}