package com.sztzjy.money_management.config.security;

import cn.hutool.extra.servlet.ServletUtil;
import com.sztzjy.money_management.config.Constant;
import com.sztzjy.money_management.config.exception.UnAuthorizedException;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.MalformedJwtException;
import io.jsonwebtoken.UnsupportedJwtException;
import io.jsonwebtoken.security.SignatureException;
import org.springframework.http.HttpStatus;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.util.AntPathMatcher;
import org.springframework.util.PathMatcher;
import org.springframework.util.StringUtils;
import org.springframework.web.filter.OncePerRequestFilter;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.List;

/**
 * 系统登陆认证拦截
 *
 * @author 陈沅
 */
public class AuthenticationFilter extends OncePerRequestFilter {
    private final PathMatcher matcher = new AntPathMatcher();

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {
        String requestRri = request.getRequestURI();
        if (requestRri.equals("/")) {
            response.sendRedirect("/doc.html");
            return;
        }
        if (!isExclude(request)) {
            System.out.println("--------------------------------->>>  " + LocalDateTime.now() + " ---ip:" + ServletUtil.getClientIP(request) + " requestUri:" + requestRri);
        }
        String token = request.getHeader(Constant.AUTHORIZATION);
        if (token == null || token.equals("undefined") || token.equals("") || token.equals("null")) {
            filterChain.doFilter(request, response);
            response.setHeader("Access-Control-Allow-Origin","*");
            return;
        }

        if (!StringUtils.startsWithIgnoreCase(token, "Bearer ")) {
            throw new UnAuthorizedException("令牌错误: 缺失Bearer..");
        }
        JwtUser currentUser;
        try {
            currentUser = com.sztzjy.money_management.config.security.TokenProvider.getJWTUser(token);
            response.setCharacterEncoding("UTF-8");
        } catch (ExpiredJwtException e1) {
            response.setStatus(HttpStatus.UNAUTHORIZED.value());
            response.getWriter().print("Token已过期");
            return;
        } catch (UnsupportedJwtException e2) {
            response.setStatus(HttpStatus.UNAUTHORIZED.value());
            response.getWriter().print("不支持的Token");
            return;
        } catch (MalformedJwtException e3) {

            response.setStatus(HttpStatus.UNAUTHORIZED.value());
            response.getWriter().print("无效的Token格式");
            return;
        } catch (SignatureException e4) {
            response.setStatus(HttpStatus.UNAUTHORIZED.value());
            response.getWriter().print("无效的Token签名");
            return;
        } catch (Exception e5) {
            response.setStatus(HttpStatus.UNAUTHORIZED.value());
            response.getWriter().print("无效的Token");
            return;
        }
        if(currentUser!=null){
            Authentication authentication = new UsernamePasswordAuthenticationToken(currentUser, token, currentUser.getAuthorities());
            SecurityContextHolder.getContext().setAuthentication(authentication);
        }
        filterChain.doFilter(request, response);
    }


    /**
     * 排除的请求路径匹配
     */
    private final static List<String> patterns = Arrays.asList(
            "/swagger-resources/**",
            "/doc.html",
            "/webjars/**",
            "/v2/**",
            "/favicon.ico",
            "/login",
            "/druid/**"
    );

    /**
     * 判断请求路径是否是在排除之外
     *
     * @param request /
     * @return true-是排除的
     */
    private boolean isExclude(HttpServletRequest request) {
        String requestURI = request.getRequestURI();
        for (String pattern : patterns) {
            if (matcher.match(pattern, requestURI)) {
                return true;
            }
        }
        return false;
    }
}