package com.sztzjy.money_management.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.sztzjy.money_management.annotation.AnonymousAccess;
import com.sztzjy.money_management.config.Constant;
import com.sztzjy.money_management.entity.*;
import com.sztzjy.money_management.entity.dto.CaseFinanceDto;
import com.sztzjy.money_management.entity.dto.CaseInvestmentPlanDto;
import com.sztzjy.money_management.mapper.*;
import com.sztzjy.money_management.service.CaseService;
import com.sztzjy.money_management.service.ComputeService;
import com.sztzjy.money_management.util.ResultEntity;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * @Author xcj
 * @Date 2024/8/12
 */
@RestController
@Api(tags = "案例相关接口")
@RequestMapping("api/case")
public class CaseController {
    @Autowired
    private CaseService caseService;
    @Autowired
    private CaseInfoMapper caseInfoMapper;
    @Autowired
    private ComCaseMapper comCaseMapper;
    @Autowired
    private ChapterMapper chapterMapper;
    @Autowired
    private InsuranceProMapper insuranceProMapper;
    @Autowired
    private ComputeService computeService;
    @Autowired
    private p2pDataMapper p2pDataMapper;
    @Autowired
    private SynthesisPlanClientMapper synthesisPlanClientMapper;
    @Autowired
    private CaseFinanceLiabilityMapper caseFinanceLiabilityMapper;
    @Autowired
    private CaseIncomeAndExpensesMapper caseIncomeAndExpensesMapper;
    @Autowired
    private CaseCashFlowMapper caseCashFlowMapper;
    @Autowired
    private CaseFinancialRatiosMapper caseFinancialRatiosMapper;

    @PostMapping("/insertCase")
    @ApiOperation("新增/编辑案例，学生端新增传isAnswer为学生答案+userId")
    private ResultEntity<HttpStatus> insertCase(@ApiParam("左侧caseInfo对象") @RequestParam(required = false) String caseInfo,  //todo 传json字符串
                                                @ApiParam("右侧考核点设置的参数对象") @RequestParam String json,
                                                @ApiParam("老师或管理员操作传master,学生操作传stu") @RequestParam String operator,
                                                @ApiParam("风险测评-风险指标/财务分析-资产负债表等等") @RequestParam(required = false) String type) {
        return caseService.insertCase(caseInfo, json, operator, type);
    }


    @PostMapping("/insertComprehensivePlanCase")
    @ApiOperation("老师端或管理员/新增编辑综合规划案例")
    private ResultEntity<HttpStatus> insertComprehensivePlanCase(@ApiParam("主表caseInfo必传") @RequestBody StuCaseDto stuCaseDto) {
        return caseService.insertComprehensivePlanCase(stuCaseDto);
    }


    @PostMapping("/getCaseFinance")
    @ApiOperation("老师端/管理员端/查询财务分析四张表")
    @AnonymousAccess
    private ResultEntity<CaseFinanceDto> getCaseFinance(@RequestParam String caseId) {
        CaseFinanceDto caseFinanceDto = new CaseFinanceDto();

        CaseFinanceLiabilityExample caseFinanceLiabilityExample = new CaseFinanceLiabilityExample();
        caseFinanceLiabilityExample.createCriteria().andCaseIdEqualTo(caseId).andUserIdIsNull().andIsAnswerEqualTo("题目答案");
        List<CaseFinanceLiability> caseFinanceLiabilities = caseFinanceLiabilityMapper.selectByExample(caseFinanceLiabilityExample);
        if (caseFinanceLiabilities != null && !caseFinanceLiabilities.isEmpty()) {
            CaseFinanceLiability caseFinanceLiability = caseFinanceLiabilities.get(0);
            caseFinanceDto.setCaseFinanceLiability(caseFinanceLiability);
        }

        CaseIncomeAndExpensesExample caseIncomeAndExpensesExample = new CaseIncomeAndExpensesExample();
        caseIncomeAndExpensesExample.createCriteria().andCaseidEqualTo(caseId).andUserIdIsNull().andIsAnswerEqualTo("题目答案");
        List<CaseIncomeAndExpenses> caseIncomeAndExpenses = caseIncomeAndExpensesMapper.selectByExample(caseIncomeAndExpensesExample);
        if (caseIncomeAndExpenses != null && !caseIncomeAndExpenses.isEmpty()) {
            CaseIncomeAndExpenses caseIncomeAndExpenses1 = caseIncomeAndExpenses.get(0);
            caseFinanceDto.setCaseIncomeAndExpenses(caseIncomeAndExpenses1);
        }

        CaseCashFlowExample example = new CaseCashFlowExample();
        example.createCriteria().andCaseidEqualTo(caseId).andUserIdIsNull().andIsAnswerEqualTo("题目答案");
        List<CaseCashFlow> caseCashFlows = caseCashFlowMapper.selectByExample(example);
        if (caseCashFlows != null && !caseCashFlows.isEmpty()) {
            CaseCashFlow caseCashFlow = caseCashFlows.get(0);
            caseFinanceDto.setCaseCashFlow(caseCashFlow);
        }

        CaseFinancialRatiosExample caseFinancialRatiosExample = new CaseFinancialRatiosExample();
        caseFinancialRatiosExample.createCriteria().andCaseidEqualTo(caseId).andUserIdIsNull().andIsAnswerEqualTo("题目答案");
        List<CaseFinancialRatiosWithBLOBs> caseFinancialRatiosWithBLOBs = caseFinancialRatiosMapper.selectByExampleWithBLOBs(caseFinancialRatiosExample);
        if (caseFinancialRatiosWithBLOBs != null && !caseFinancialRatiosWithBLOBs.isEmpty()) {
            CaseFinancialRatiosWithBLOBs caseFinancialRatios = caseFinancialRatiosWithBLOBs.get(0);
            caseFinanceDto.setCaseFinancialRatios(caseFinancialRatios);
        }
        return new ResultEntity<CaseFinanceDto>(caseFinanceDto);
    }


    @PostMapping("/getInsurancePro")
    @ApiOperation("查询保险规划产品名称")
    @AnonymousAccess
    private ResultEntity<List<Map<String, String>>> getInsurancePro(@RequestParam String type) {
        return new ResultEntity<>(insuranceProMapper.getInsurancePro(type));
    }

    @PostMapping("/insertOrUpdateInvestmentPlan")
    @ApiOperation("新增/编辑案例，投资规划")
    private ResultEntity<HttpStatus> insertOrUpdateInvestmentPlan(@ApiParam("老师或管理员新增修改不传userId") @RequestBody CaseInvestmentPlanDto caseInvestmentPlanDto) {
        return caseService.insertOrUpdateInvestmentPlan(caseInvestmentPlanDto);
    }


    @PostMapping("/deleteCase")
    @ApiOperation("删除案例题")
    private ResultEntity<HttpStatus> deleteCase(@RequestParam String caseId,
                                                @ApiParam("管理员/老师") @RequestParam String role) {
        CaseInfo caseInfo = caseInfoMapper.selectByPrimaryKey(caseId);
        if (role.equals("老师")) {
            if (caseInfo.getSource().equals("999999999")) {
                return new ResultEntity<HttpStatus>(HttpStatus.BAD_REQUEST, "内置题目，无法删除");
            }
            caseInfoMapper.deleteByPrimaryKey(caseId);

            SynthesisPlanClientExample example = new SynthesisPlanClientExample();
            example.createCriteria().andCaseidEqualTo(caseId);
            synthesisPlanClientMapper.deleteByExample(example);

            ComCaseExample example1 = new ComCaseExample();
            example1.createCriteria().andCaseidEqualTo(caseId);
            comCaseMapper.deleteByExample(example1);
            return new ResultEntity<HttpStatus>(HttpStatus.OK, "删除成功");
        }
        if (role.equals("管理员")) {
            caseInfoMapper.deleteByPrimaryKey(caseId);

            SynthesisPlanClientExample example = new SynthesisPlanClientExample();
            example.createCriteria().andCaseidEqualTo(caseId);
            synthesisPlanClientMapper.deleteByExample(example);

            ComCaseExample example1 = new ComCaseExample();
            example1.createCriteria().andCaseidEqualTo(caseId);
            comCaseMapper.deleteByExample(example1);
            return new ResultEntity<HttpStatus>(HttpStatus.OK, "删除成功");
        }
        return null;
    }


    @PostMapping("/ZHGHType")
    @ApiOperation("综合规划下方展示类型")
    @AnonymousAccess
    private ResultEntity<String> ZHGHType(@RequestParam String caseID) {
        ComCaseExample example = new ComCaseExample();
        example.createCriteria().andCaseidEqualTo(caseID);
        List<ComCase> comCases = comCaseMapper.selectByExample(example);
        if (comCases.isEmpty()) {
            return null;
        }
        return new ResultEntity<>(comCases.get(0).getManageMoneyMattersType());
    }

    @AnonymousAccess
    @PostMapping("/getExamPoint")
    @ApiOperation("展示考核点信息/老师端/学生实验实训")
    private ResultEntity<Object> getExamPoint(@RequestParam String caseID,
                                              @ApiParam("老师端不传") @RequestParam(required = false) String userId,
                                              @ApiParam("学生答案/题目答案") @RequestParam String isAnswer,
                                              @ApiParam("理财类型/报告展示传 报告") @RequestParam String type) {
        return caseService.getExamPoint(caseID, userId, isAnswer, type);
    }

    @PostMapping("/getCaseInfoById")
    @ApiOperation("展示单个案例客户信息")
    @AnonymousAccess
    private ResultEntity<CaseInfo> getCaseInfoById(@RequestParam String caseID) {
        CaseInfo caseInfo = caseInfoMapper.selectByPrimaryKey(caseID);
        return new ResultEntity<CaseInfo>(caseInfo);
    }

    @GetMapping("/getCaseListByAdmin")
    @ApiOperation("展示内置所有案例客户信息/实验实训使用")
    @AnonymousAccess
    private ResultEntity<List<CaseInfo>> getCaseListByAdmin() {
        CaseInfoExample example = new CaseInfoExample();
        example.createCriteria().andSourceEqualTo(Constant.BUILT_IN_SCHOOL_ID);
        example.setOrderByClause("add_time");
        List<CaseInfo> caseInfoList = caseInfoMapper.selectByExampleWithBLOBs(example);
        return new ResultEntity<List<CaseInfo>>(caseInfoList);
    }


    @PostMapping("/commitTraining")
    @ApiOperation("学生端提交实验实训")
    @AnonymousAccess
    private void commitTraining(@ApiParam("userId，caseId,isAnswer为学生答案") @RequestParam String json,
                                @ApiParam("案例类型") @RequestParam String type,
                                @RequestParam String trainingId) {
        caseService.commitTraining(json, type, trainingId);
    }

//    @PostMapping("/updateExamPoint")
//    @ApiOperation("修改考核点内容")
//    @AnonymousAccess
//    private ResultEntity<HttpStatus> updateExamPoint(@RequestParam String json,
//                                                     @ApiParam("值为考核点下拉框展示的内容") @RequestParam String type) {
//        return caseService.updateExamPoint(json, type);
//    }

//    @PostMapping("/updateCaseInfoById")
//    @ApiOperation("修改客户信息")
//    @AnonymousAccess
//    private ResultEntity<HttpStatus> updateCaseInfoById(@RequestBody CaseInfo caseInfo) {
//        if (StringUtils.isBlank(caseInfo.getCaseid())) {
//            return new ResultEntity<HttpStatus>(HttpStatus.BAD_REQUEST, "缺少主键ID，无法修改");
//        }
//        try {
//            caseInfoMapper.updateByPrimaryKeySelective(caseInfo);
//            return new ResultEntity<HttpStatus>(HttpStatus.OK, "编辑成功");
//        } catch (Exception e) {
//            e.printStackTrace();
//            return new ResultEntity<HttpStatus>(HttpStatus.INTERNAL_SERVER_ERROR, "服务器错误，请练习管理员！");
//        }
//    }

    @PostMapping("/getCaseInfoList")
    @ApiOperation("案例列表展示")
    @AnonymousAccess
    private ResultEntity<PageInfo<CaseInfo>> getCaseInfoList(@RequestParam Integer index,
                                                             @RequestParam Integer size,
                                                             @RequestParam(required = false) String model,
                                                             @RequestParam(required = false) String schoolId,
                                                             @RequestParam(required = false) String keyWord,
                                                             @RequestParam(required = false) String type) {
        PageHelper.startPage(index, size);
        List<CaseInfo> caseInfoList = caseInfoMapper.getCaseInfoList(model, schoolId, keyWord, type);
        PageInfo pageInfo = new PageInfo(caseInfoList);
        return new ResultEntity<PageInfo<CaseInfo>>(pageInfo);
    }


    @PostMapping("/getCaseInfoByChapterId")
    @ApiOperation("根据章节查询案例")
    @AnonymousAccess
    private ResultEntity<List<CaseInfo>> getCaseInfoByChapterId(@RequestParam String type) {
        CaseInfoExample example = new CaseInfoExample();
        example.createCriteria().andManageMoneyMattersTypeEqualTo(type);
        List<CaseInfo> caseInfoList = caseInfoMapper.selectByExampleWithBLOBs(example);
        return new ResultEntity<>(caseInfoList);
    }


    @PostMapping("/insertKnowledge")
    @ApiOperation("知识点新增/编辑")
    @AnonymousAccess
    private ResultEntity<HttpStatus> insertKnowledge(@ApiParam("请求头可能装不下,放对象,传ID和knowledge") @RequestBody Chapter chapter) {
        Chapter newChapter = chapterMapper.selectByPrimaryKey(chapter.getChapterId());
        if (newChapter == null) {
            return new ResultEntity<HttpStatus>(HttpStatus.INTERNAL_SERVER_ERROR, "服务器异常，请联系管理员！");
        }
        newChapter.setKnowledge(chapter.getKnowledge());
        int i = chapterMapper.updateByPrimaryKeySelective(newChapter);
        if (i <= 0) {
            return new ResultEntity<HttpStatus>(HttpStatus.INTERNAL_SERVER_ERROR, "服务器异常，请联系管理员！");
        }
        return new ResultEntity<HttpStatus>(HttpStatus.OK, "新增成功！");
    }


    @PostMapping("/getKnowledge")
    @ApiOperation("知识点展示")
    @AnonymousAccess
    private ResultEntity<String> getKnowledge(@RequestParam String chapterID) {
        Chapter chapter = chapterMapper.selectByPrimaryKey(chapterID);
        if (chapter == null) {
            return null;
        }
        return new ResultEntity<>(chapter.getKnowledge());
    }

    @PostMapping("/getAnnualPremium")
    @ApiOperation("年缴保费计算")
    @AnonymousAccess
    private ResultEntity<BigDecimal> getInsuranceExpense(@ApiParam("需要年龄,性别,保险年份,缴费方式,保险金额,InsuranceProId") @RequestBody CaseInvestmentPlanInsuranceProductInfo productInfo) {
        return computeService.getAnnualPremium(productInfo);
    }


    @PostMapping("getP2PProductById")
    @ApiOperation("根据ID查询单个P2P产品")
    @AnonymousAccess
    public ResultEntity<p2pData> getP2PProductById(@RequestParam String id) {
        p2pData p2pProducts = p2pDataMapper.selectByPrimaryKey(id);
        return new ResultEntity<>(p2pProducts);
    }


}
